@@ -30,7 +30,7 @@ class EventHandler(CachedPropertiesMixin, metaclass=StructMeta):
     durations_dict : dict
         Dictionary with the information abaout all added durations of a process.
 
-    See also
+    See Also
     --------
     Events
     add_event
@@ -70,7 +70,7 @@ class EventHandler(CachedPropertiesMixin, metaclass=StructMeta):
 
     def add_event(
             self, name, parameter_path, state, time=0.0, entry_index=None,
-            dependencies=None, factors=None, 
+            dependencies=None, factors=None,
         ):
         """Factory function for creating and adding events.
 
@@ -92,7 +92,7 @@ class EventHandler(CachedPropertiesMixin, metaclass=StructMeta):
         CADETProcessError
             If EventPerformer is not found in EventHandler
 
-        See also
+        See Also
         --------
         Event
         remove_event
@@ -107,7 +107,7 @@ class EventHandler(CachedPropertiesMixin, metaclass=StructMeta):
 
         self._events.append(evt)
         super().__setattr__(name, evt)
-        
+
         if dependencies is not None:
             self.add_event_dependency(evt.name, dependencies, factors)
 
@@ -130,7 +130,7 @@ class EventHandler(CachedPropertiesMixin, metaclass=StructMeta):
         ----
         !!! Check remove_event_dependencies
 
-        See also
+        See Also
         --------
         add_event
         remove_event_dependency
@@ -159,7 +159,7 @@ class EventHandler(CachedPropertiesMixin, metaclass=StructMeta):
         CADETProcessError
             If Duration already exists.
 
-        See also
+        See Also
         --------
         durations
         remove_duration
@@ -188,7 +188,7 @@ class EventHandler(CachedPropertiesMixin, metaclass=StructMeta):
         CADETProcessError
             If Duration is not found.
 
-        See also
+        See Also
         --------
         Duration
         add_duration
@@ -222,7 +222,7 @@ class EventHandler(CachedPropertiesMixin, metaclass=StructMeta):
         event object.
 
         Parameters
-        ---------
+        ----------
         dependent_event : str
             Name of the event whose value will depend on other events.
         independent_events : list
@@ -239,7 +239,7 @@ class EventHandler(CachedPropertiesMixin, metaclass=StructMeta):
             combined_evt_dur dictionary.
             If length of factors does not equal length of independent events.
 
-        See also
+        See Also
         --------
         Event
         add_dependency
@@ -278,7 +278,7 @@ class EventHandler(CachedPropertiesMixin, metaclass=StructMeta):
         dependency.
 
         Parameters
-        ---------
+        ----------
         dependent_event : str
             Name of the event whose value will depend on other events.
         independent_events : list
@@ -291,8 +291,8 @@ class EventHandler(CachedPropertiesMixin, metaclass=StructMeta):
             If one or more independent event is not in list events and
             durations.
 
-        See also:
-        ---------
+        See Also
+        --------
         remove_dependecy
         Event
         """
@@ -532,22 +532,39 @@ class EventHandler(CachedPropertiesMixin, metaclass=StructMeta):
 
     def plot_events(self):
         """Plot parameter state as function of time.
+
+        Parameters
+        ----------
+        ax : Axes
+            Axes to plot on.
+
+        Returns
+        -------
+        ax : Axes
+            Axes with plot of parameter state.
         """
         time = np.linspace(0, self.cycle_time, 1001)
+
+        axs = []
+
         for parameter, tl in self.parameter_timelines.items():
-            y = tl.value(time)
-            
             fig, ax = plotting.setup_figure()
-        
+
+            y = tl.value(time)
+
             layout = plotting.Layout()
             layout.title = str(parameter)
             layout.x_label = '$time~/~min$'
             layout.y_label = '$state$'
-        
+
             ax.plot(time/60, y)
 
-            plotting.set_layout(fig, ax, layout)        
-        
+            plotting.set_layout(ax, layout)
+            
+            axs.append(ax)
+
+        return axs
+
 class Event():
     """Class for defining dynamic changes.
 
@@ -581,7 +598,7 @@ class Event():
         If performner does not have attribute.
         If state is not valid for attribute.
 
-    See also:
+    See Also:
     ---------
     EventHandler
     Duration
@@ -772,8 +789,8 @@ class Event():
 
     @time.setter
     def time(self, time):
-        if not isinstance(time, (int, float, np.int64, np.float64)):
-            raise TypeError("Expected {}".format(float))
+        if not np.isscalar(time):
+            raise TypeError("Expected scalar value")
 
         if self.isIndependent:
             self._time = time
