@@ -29,7 +29,7 @@ class Fractionator(EventHandler):
         Object of the class TimeSignal, array with the concentration over time
         for a simulated process.
     """
-    
+
     name = String(default='Fractionator')
     performance_keys = [
         'mass', 'concentration', 'purity', 'recovery',
@@ -187,9 +187,9 @@ class Fractionator(EventHandler):
         """
         return self.chromatograms[0].time
 
-    @plotting.save_fig
+    @plotting.create_and_save_figure
     def plot_fraction_signal(
-            self, start=0, end=None, index=0, secondary_axis=None
+            self, start=0, end=None, index=0, secondary_axis=None, ax=None
             ):
         """Plot the signal without the waste fractions.
 
@@ -197,14 +197,17 @@ class Fractionator(EventHandler):
         ----------
         start : float, optional
             Start time of the plot. The default is 0.
-        end : TYPE, optional
+        end : float, optional
             End time of the plot. The default is None.
         index : int, optional
             Chromatogram index. The default is 0.
+        ax : Axes
+            Axes to plot on.
 
         Returns
         -------
-        fig, axs.
+        ax : Axes
+            Axes with plot of parameter state.
 
         See also
         --------
@@ -215,9 +218,8 @@ class Fractionator(EventHandler):
         time_line = self.performer_timelines['fractionation_states'][chrom.name]
         x = chrom.time/60
         y = chrom.signal
-        ymax = 1.1*np.max(chrom.signal)
-
-        fig, ax = plotting.setup_figure()
+        y_min = np.min(chrom.signal)
+        y_max = 1.1*np.max(chrom.signal)
 
         ax.plot(x,y)
 
@@ -235,7 +237,7 @@ class Fractionator(EventHandler):
                 fill_regions.append(plotting.FillRegion(
                     start=sec.start/60,
                     end=sec.end/60,
-                    y_max=ymax,
+                    y_max=y_max,
                     color_index=color_index,
                     text=text
                     )
@@ -247,7 +249,7 @@ class Fractionator(EventHandler):
             fill_regions.append(plotting.FillRegion(
                 start=sec.start/60,
                 end=sec.end/60,
-                y_max=ymax,
+                y_max=y_max,
                 color_index=-1,
                 text='W'
                 )
@@ -258,12 +260,12 @@ class Fractionator(EventHandler):
         layout = plotting.Layout()
         layout.x_label = '$time~/~min$'
         layout.y_label = '$c~/~mM$'
-        layout.xlim = (start, end)
-        layout.ylim = (0, ymax)
+        layout.x_lim = (start, end)
+        layout.y_lim = (y_min, y_max)
 
-        plotting.set_layout(fig, ax, layout)
+        plotting.set_layout(ax, layout)
 
-        return fig, ax
+        return ax
 
     @property
     def fractionation_states(self):
@@ -479,7 +481,7 @@ class Fractionator(EventHandler):
         local purity profile is higher than the purity required.
 
         Parameters
-        -----------
+        ----------
         purity_required : float or list of floats
             Minimum purity required for the components in the fractionation
 
