@@ -13,12 +13,17 @@ class BindingBaseClass(metaclass=StructMeta):
 
     Attributes
     ----------
-    n_comp : UnsignedInteger
-        number of components.
-    parameters : dict
-        dict with parameter values.
     name : String
         name of the binding model.
+    component_system : ComponentSystem
+        system of components.
+    n_comp : int
+        number of components.
+    is_kinetic : bool
+        If False, adsorption is assumed to be in rapid equilibriu. 
+        The default is True.
+    parameters : dict
+        dict with parameter values.
     """
     name = String()
     is_kinetic = Bool(default=True)
@@ -68,14 +73,12 @@ class BindingBaseClass(metaclass=StructMeta):
 
 
     def __repr__(self):
-        return '{}(n_comp={}, name=\'{}\')'.format(
-            self.__class__.__name__, self.n_comp, self.name
-        )
+        return f"{self.__class__.__name__}(\
+            component_system={self.component_system}, name={self.name})')"
 
     def __str__(self):
         return self.name
 
-
 class NoBinding(BindingBaseClass):
     """Dummy class for units that do not experience binging behavior.
 
@@ -84,7 +87,6 @@ class NoBinding(BindingBaseClass):
     def __init__(self, *args, **kwargs):
         super().__init__(ComponentSystem(), name='NoBinding')
 
-
 class Linear(BindingBaseClass):
     """Parameters for Linear binding model.
 
@@ -122,6 +124,28 @@ class Langmuir(BindingBaseClass):
         'adsorption_rate', 'desorption_rate', 'capacity'
     ]
 
+class LangmuirLDF(BindingBaseClass):
+    """Parameters for Multi Component Langmuir binding model.
+
+    Attributes
+    ----------
+    equilibrium_constant : list of unsigned floats. Length depends on n_comp.
+        Adsorption rate constants.
+    driving_force_coefficient : list of unsigned floats. Length depends on n_comp.
+        Desorption rate constants
+    capacity : list of unsigned floats. Length depends on n_comp.
+        Maximum adsorption capacities.
+    """
+    equilibrium_constant = DependentlySizedUnsignedList(dep=('n_comp', 'n_states'))
+    driving_force_coefficient = DependentlySizedUnsignedList(dep=('n_comp', 'n_states'), default=1)
+    capacity = DependentlySizedUnsignedList(dep=('n_comp', 'n_states'))
+
+    _parameter_names = BindingBaseClass._parameter_names + [
+        'equilibrium_constant',
+        'driving_force_coefficient',
+        'capacity',
+    ]
+
 class BiLangmuir(BindingBaseClass):
     """Parameters for Multi Component Bi-Langmuir binding model.
 
@@ -140,11 +164,12 @@ class BiLangmuir(BindingBaseClass):
     n_states = UnsignedInteger()
 
     _parameter_names = BindingBaseClass._parameter_names + [
-            'adsorption_rate',
-            'desorption_rate',
-            'capacity',
-            'n_states'
-        ]
+        'adsorption_rate',
+        'desorption_rate',
+        'capacity',
+        'n_states'
+    ]
+
     def __init__(self, *args, n_states=2, **kwargs):
         self.n_states = n_states
 
@@ -154,6 +179,60 @@ class BiLangmuir(BindingBaseClass):
     def n_total_states(self):
         return self.n_comp * self.n_states
 
+class BiLangmuirLDF(BindingBaseClass):
+    """Parameters for Multi Component Bi-Langmuir binding model.
+
+    Attributes
+    ----------
+    equilibrium_constant : list of unsigned floats. Length depends on n_comp.
+        Adsorption rate constants.
+    driving_force_coefficient : list of unsigned floats. Length depends on n_comp.
+        Desorption rate constants
+    capacity : list of unsigned floats. Length depends on n_comp.
+        Maximum adsorption capacities.
+    """
+    equilibrium_constant = DependentlySizedUnsignedList(dep=('n_comp', 'n_states'))
+    driving_force_coefficient = DependentlySizedUnsignedList(dep=('n_comp', 'n_states'), default=1)
+    capacity = DependentlySizedUnsignedList(dep=('n_comp', 'n_states'))
+    n_states = UnsignedInteger()
+
+    _parameter_names = BindingBaseClass._parameter_names + [
+        'equilibrium_constant',
+        'driving_force_coefficient',
+        'capacity',
+        'n_states'
+    ]
+
+    def __init__(self, *args, n_states=2, **kwargs):
+        self.n_states = n_states
+
+        super().__init__(*args, **kwargs)
+
+    @property
+    def n_total_states(self):
+        return self.n_comp * self.n_states
+
+class FreundlichLDF(BindingBaseClass):
+    """Parameters for the Freundlich isotherm model.
+
+    Attributes
+    ----------
+    driving_force_coefficient : list of unsigned floats.
+        Adsorption rate constants. Length depends on n_comp.
+    freundlich_coefficient : list of unsigned floats.
+        Freundlich coefficient for each component. Length depends on n_comp.
+    exponent : list of unsigned floats.
+        Exponent for each component. Length depends on n_comp.
+    """
+    driving_force_coefficient = DependentlySizedUnsignedList(dep='n_comp')
+    freundlich_coefficient = DependentlySizedUnsignedList(dep='n_comp')
+    exponent = DependentlySizedUnsignedList(dep='n_comp')
+
+    _parameter_names = BindingBaseClass._parameter_names + [
+        'driving_force_coefficient',
+        'freundlich_coefficient',
+        'exponent',
+    ]
 
 class StericMassAction(BindingBaseClass):
     """Parameters for Steric Mass Action Law binding model.
@@ -622,3 +701,116 @@ class Saska(BindingBaseClass):
     _parameter_names = BindingBaseClass._parameter_names + [
         'henry_const', 'quadratic_factor'
     ]
+
+class GeneralizedIonExchange(BindingBaseClass):
+    """Generalized Ion Exchange isotherm model.
+
+    Attributes
+    ----------
+    adsorption_rate : list of unsigned floats. Length depends on n_comp.
+        Adsorption rate constants.
+    adsorption_rate_linear : list of unsigned floats. Length depends on n_comp.
+        Linear dependence coefficient of adsorption rate on modifier component
+    adsorption_rate_quadratic : list of unsigned floats. Length depends on n_comp.
+        Quadratic dependence coefficient of adsorption rate on modifier component.
+    adsorption_rate_cubic : list of unsigned floats. Length depends on n_comp.
+        Cubic dependence coefficient of adsorption rate on modifier component.
+    adsorption_rate_salt : list of unsigned floats. Length depends on n_comp.
+        Salt coefficient of adsorption rate;
+        difference of water-protein and salt-protein interactions.
+    adsorption_rate_protein : list of unsigned floats. Length depends on n_comp.
+        Protein coefficient of adsorption rate;
+        difference of water-protein and protein-protein interactions.
+    desorption_rate : list of unsigned floats. Length depends on n_comp.
+        Desorption rate constants.
+    desorption_rate_linear : list of unsigned floats. Length depends on n_comp.
+        Linear dependence coefficient of desorption rate on modifier component.
+    desorption_rate_quadratic : list of unsigned floats. Length depends on n_comp.
+        Quadratic dependence coefficient of desorption rate on modifier component.
+    desorption_rate_cubic : list of unsigned floats. Length depends on n_comp.
+        Cubic dependence coefficient of desorption rate on modifier component.
+    desorption_rate_salt : list of unsigned floats. Length depends on n_comp.
+        Salt coefficient of desorption rate;
+        difference of water-protein and salt-protein interactions.
+    desorption_rate_protein : list of unsigned floats. Length depends on n_comp.
+        Protein coefficient of desorption rate;
+        difference of water-protein and protein-protein interactions
+    characteristic_charge : list of unsigned floats. Length depends on n_comp.
+        The characteristic charge of the protein: The number sites v that
+        protein interacts on the resin surface.
+    characteristic_charge_linear : list of unsigned floats. Length depends on n_comp.
+        Linear dependence coefficient of characteristic charge on modifier component.
+    characteristic_charge_quadratic : list of unsigned floats. Length depends on n_comp.
+        Quadratic dependence coefficient of characteristic charge on modifier component.
+    characteristic_charge_cubic : list of unsigned floats. Length depends on n_comp.
+        Cubic dependence coefficient of characteristic charge on modifier component .
+    characteristic_charge_breaks : list of unsigned floats. Length depends on n_comp.
+        Cubic dependence coefficient of characteristic charge on modifier component .
+    steric_factor : list of unsigned floats. Length depends on n_comp.
+        Steric factors of the protein: The number of sites o on the surface
+        that are shileded by the protein and prevented from exchange with salt
+        counterions in solution.
+    capacity : unsigned float.
+        Stationary phase capacity (monovalent salt counterions); The total
+        number of binding sites available on the resin surface.
+    reference_liquid_phase_conc : unsigned float.
+        Reference liquid phase concentration (optional, default value = 1.0).
+    reference_solid_phase_conc : unsigned float.
+        Reference liquid phase concentration (optional, default value = 1.0).
+    """
+    adsorption_rate = DependentlySizedUnsignedList(dep='n_comp')
+    adsorption_rate_linear = DependentlySizedUnsignedList(dep='n_comp')
+    adsorption_rate_quadratic = DependentlySizedUnsignedList(dep='n_comp', default=0)
+    adsorption_rate_cubic = DependentlySizedUnsignedList(dep='n_comp', default=0)
+    adsorption_rate_salt = DependentlySizedUnsignedList(dep='n_comp', default=0)
+    adsorption_rate_protein = DependentlySizedUnsignedList(dep='n_comp', default=0)
+    desorption_rate = DependentlySizedUnsignedList(dep='n_comp', default=1)
+    desorption_rate_linear = DependentlySizedUnsignedList(dep='n_comp', default=0)
+    desorption_rate_quadratic = DependentlySizedUnsignedList(dep='n_comp', default=0)
+    desorption_rate_cubic = DependentlySizedUnsignedList(dep='n_comp', default=0)
+    desorption_rate_salt = DependentlySizedUnsignedList(dep='n_comp', default=0)
+    desorption_rate_protein = DependentlySizedUnsignedList(dep='n_comp', default=0)
+    characteristic_charge = DependentlySizedUnsignedList(dep='_break_length', default=1)
+    characteristic_charge_linear = DependentlySizedUnsignedList(dep='characteristic_charge', default=0)
+    characteristic_charge_quadratic = DependentlySizedUnsignedList(dep='characteristic_charge', default=0)
+    characteristic_charge_cubic = DependentlySizedUnsignedList(dep='characteristic_charge', default=0)
+    characteristic_charge_breaks = DependentlySizedUnsignedList(dep='_break_length', default=0)
+    steric_factor = DependentlySizedUnsignedList(dep='n_comp')
+    capacity = UnsignedFloat()
+    reference_liquid_phase_conc = UnsignedFloat()
+    reference_solid_phase_conc = UnsignedFloat()
+
+    _parameter_names = BindingBaseClass._parameter_names + [
+        'adsorption_rate',
+        'adsorption_rate_linear',
+        'adsorption_rate_quadratic',
+        'adsorption_rate_cubic',
+        'adsorption_rate_salt',
+        'adsorption_rate_protein',
+        'desorption_rate',
+        'desorption_rate_linear',
+        'desorption_rate_quadratic',
+        'desorption_rate_cubic',
+        'desorption_rate_salt',
+        'desorption_rate_protein',
+        'characteristic_charge',
+        'characteristic_charge_linear',
+        'characteristic_charge_quadratic',
+        'characteristic_charge_cubic',
+        'characteristic_charge_breaks',
+        'steric_factor',
+        'capacity',
+        'reference_liquid_phase_conc',
+        'reference_solid_phase_conc',
+    ]
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+    @property
+    def _multiples(self):
+        pass
+
+    @property
+    def _break_length(self):
+        return len(self.characteristic_charge) + 1
