@@ -289,6 +289,11 @@ class Cadet(SolverBase):
         cadet.root = self.get_process_config(process)
         cadet.filename = file_path
         cadet.save()
+        
+    def run_h5(self, file_path):
+        cadet = CadetAPI()
+        cadet.filename = file_path
+        cadet.run(timeout=self.timeout)
 
     def load_from_h5(self, file_path):
         cadet = CadetAPI()
@@ -324,6 +329,13 @@ class Cadet(SolverBase):
 
         return config
 
+    def load_simulation_results(self, process, file_path):
+        cadet = self.load_from_h5(file_path)
+        results = self.get_simulation_results(process, cadet)
+
+        return results
+
+
     def get_simulation_results(
         self,
         process,
@@ -383,61 +395,61 @@ class Cadet(SolverBase):
                 unit_coordinates = cadet.root.output.coordinates[unit_index].copy()
                 particle_coordinates = \
                     unit_coordinates.pop('particle_coordinates_000', None)
-                        
+
                 for cycle in range(process._n_cycles):
                     start = cycle * (len(process.time) -1)
                     end = (cycle + 1) * (len(process.time) - 1) + 1
-                
+
                     if 'solution_inlet' in unit_solution.keys():
                         sol_inlet = unit_solution.solution_inlet[start:end,:]
                         solution[unit.name]['inlet'].append(
                             SolutionIO(unit.component_system, time, sol_inlet)
                         )
-                    
+
                     if 'solution_outlet' in unit_solution.keys():
                         sol_outlet = unit_solution.solution_outlet[start:end,:]
                         solution[unit.name]['outlet'].append(
                             SolutionIO(unit.component_system, time, sol_outlet)
                         )
-    
+
                     if 'solution_bulk' in unit_solution.keys():
                         sol_bulk = unit_solution.solution_bulk[start:end,:]
                         solution[unit.name]['bulk'].append(
                             SolutionBulk(
-                                unit.component_system, time, sol_bulk, 
+                                unit.component_system, time, sol_bulk,
                                 **unit_coordinates
                             )
                         )
-                        
+
                     if 'solution_particle' in unit_solution.keys():
                         sol_particle = unit_solution.solution_particle[start:end,:]
                         solution[unit.name]['particle'].append(
                             SolutionParticle(
                                 unit.component_system, time, sol_particle,
-                                **unit_coordinates, 
-                                particle_coordinates=particle_coordinates
-                            )
-                        )
-    
-                    if 'solution_solid' in unit_solution.keys():
-                        sol_solid = unit_solution.solution_solid[start:end,:]
-                        solution[unit.name]['solid'].append(
-                            SolutionSolid(
-                                unit.component_system, unit.binding_model.n_states, 
-                                time, sol_solid, 
                                 **unit_coordinates,
                                 particle_coordinates=particle_coordinates
                             )
                         )
-                        
+
+                    if 'solution_solid' in unit_solution.keys():
+                        sol_solid = unit_solution.solution_solid[start:end,:]
+                        solution[unit.name]['solid'].append(
+                            SolutionSolid(
+                                unit.component_system, unit.binding_model.n_states,
+                                time, sol_solid,
+                                **unit_coordinates,
+                                particle_coordinates=particle_coordinates
+                            )
+                        )
+
                     if 'solution_volume' in unit_solution.keys():
                         sol_volume = unit_solution.solution_volume[start:end,:]
                         solution[unit.name]['volume'].append(
                             SolutionVolume(unit.component_system, time, sol_volume)
                         )
-                        
+
             solution = Dict(solution)
-            
+
             system_state = {
                 'state': cadet.root.output.last_state_y,
                 'state_derivative': cadet.root.output.last_state_ydot
@@ -446,7 +458,7 @@ class Cadet(SolverBase):
             chromatograms = [
                 Chromatogram(
                     process.time, solution[chrom.name].outlet[-1].solution,
-                    process.flow_rate_timelines[chrom.name].total,
+                    process.flow_rate_timelines[chrom.name].total_in,
                     name=chrom.name
                 )
                 for chrom in process.flow_sheet.chromatogram_sinks
@@ -505,7 +517,7 @@ class Cadet(SolverBase):
         model_connections = Dict()
         if self._force_constant_flow_rate:
             model_connections['CONNECTIONS_INCLUDE_DYNAMIC_FLOW'] = 0
-        else: 
+        else:
             model_connections['CONNECTIONS_INCLUDE_DYNAMIC_FLOW'] = 1
 
         index = 0
@@ -576,7 +588,7 @@ class Cadet(SolverBase):
         """Helper function for getting unit index in CADET format unit_xxx.
 
         Parameters
-        -----------
+        ----------
         process : Process
             process to be simulated
         unit : UnitOperation
@@ -648,17 +660,17 @@ class Cadet(SolverBase):
 
         if not isinstance(unit.bulk_reaction_model, NoReaction):
             parameters = self.get_reaction_config(unit.bulk_reaction_model)
+            unit_config['reaction_model'] = parameters['REACTION_MODEL']
+            unit_config['reaction_bulk'] = parameters
+
+        if not isinstance(unit.particle_reaction_model, NoReaction):
+            parameters = self.get_reaction_config(unit.particle_reaction_model)
             if isinstance(unit, LumpedRateModelWithoutPores):
                 unit_config['reaction_model'] = parameters['REACTION_MODEL']
                 unit_config['reaction'] = parameters
             else:
-                unit_config['reaction_model'] = parameters['REACTION_MODEL']
-                unit_config['reaction_bulk'] = parameters
-
-        if not isinstance(unit.particle_reaction_model, NoReaction):
-            parameters = self.get_reaction_config(unit.particle_reaction_model)
-            unit_config['reaction_model_particle'] = parameters['REACTION_MODEL']
-            unit_config['reaction_particle'].update(parameters)
+                unit_config['reaction_model_particle'] = parameters['REACTION_MODEL']
+                unit_config['reaction_particle'].update(parameters)
 
         if isinstance(unit, Source):
             unit_config['sec_000']['const_coeff'] = unit.c[:,0]
@@ -831,7 +843,7 @@ class ModelSolverParametersGroup(ParametersGroup):
     SCHUR_SAFETY = UnsignedFloat(default=1e-8)
     LINEAR_SOLUTION_MODE = UnsignedInteger(default=2, ub=2)
     _parameters = [
-        'GS_TYPE', 
+        'GS_TYPE',
         'MAX_KRYLOV',
         'MAX_RESTARTS',
         'SCHUR_SAFETY',
@@ -994,6 +1006,15 @@ adsorption_parameters_map = {
             'MCL_QMAX': 'capacity'
         },
     },
+    'LangmuirLDF': {
+        'name': 'MULTI_COMPONENT_LANGMUIR_LDF',
+        'parameters': {
+            'IS_KINETIC' : 'is_kinetic',
+            'MCLDF_KEQ': 'equilibrium_constant',
+            'MCLDF_KKIN': 'driving_force_coefficient',
+            'MCLDF_QMAX': 'capacity'
+        },
+    },
     'BiLangmuir': {
         'name': 'MULTI_COMPONENT_BILANGMUIR',
         'parameters': {
@@ -1003,6 +1024,24 @@ adsorption_parameters_map = {
             'MCBL_QMAX': 'capacity'
         },
     },
+    'BiLangmuirLDF': {
+        'name': 'MULTI_COMPONENT_BILANGMUIR_LDF',
+        'parameters': {
+            'IS_KINETIC' : 'is_kinetic',
+            'MCBLDF_KEQ': 'equilibrium_constant',
+            'MCBLDF_KKIN': 'driving_force_coefficient',
+            'MCBLDF_QMAX': 'capacity'
+        },
+    },
+    'FreundlichLDF': {
+        'name': 'FREUNDLICH_LDF',
+        'parameters': {
+            'IS_KINETIC' : 'is_kinetic',
+            'FLDF_KKIN': 'driving_force_coefficient',
+            'FLDF_KF': 'freundlich_coefficient',
+            'FLDF_N': 'exponent'
+        },
+    },
     'StericMassAction': {
         'name': 'STERIC_MASS_ACTION',
         'parameters': {
@@ -1244,12 +1283,12 @@ class UnitReturnParametersGroup(ParametersGroup):
         'write_coordinates',
         'write_solution_inlet', 'write_solution_outlet', 'write_solution_bulk',
         'write_solution_particle', 'write_solution_solid', 'write_solution_flux',
-        'write_solution_volume', 
-        'write_soldot_inlet', 'write_soldot_outlet', 'write_soldot_bulk', 
-        'write_soldot_particle', 'write_soldot_solid', 'write_soldot_flux', 
-        'write_soldot_volume', 
-        'write_sens_inlet', 'write_sens_outlet', 'write_sens_bulk', 
-        'write_sens_particle', 'write_sens_solid', 'write_sens_flux', 
+        'write_solution_volume',
+        'write_soldot_inlet', 'write_soldot_outlet', 'write_soldot_bulk',
+        'write_soldot_particle', 'write_soldot_solid', 'write_soldot_flux',
+        'write_soldot_volume',
+        'write_sens_inlet', 'write_sens_outlet', 'write_sens_bulk',
+        'write_sens_particle', 'write_sens_solid', 'write_sens_flux',
         'write_sens_volume',
         'write_sensdot_inlet', 'write_sensdot_outlet', 'write_sensdot_bulk',
         'write_sensdot_particle', 'write_sensdot_solid', 'write_sensdot_flux',
@@ -1269,4 +1308,3 @@ class SensitivityParametersGroup(ParametersGroup):
     """
     nsens = UnsignedInteger(default=0)
     sens_method = Switch(default='ad1', valid=['ad1'])
-    
\ No newline at end of file
